import {
  ClassType,
} from '@overflow/commons/core/type';

import {
  AnnotatedElement,
  Annotation,
 } from '@overflow/commons/core/reflect';

export abstract class AccessibleObject implements AnnotatedElement {
  private _annotations: Map<any, Annotation>;

  protected constructor() {
    this._annotations = new Map();
  }

  public _addAnnotation<AnnotationType extends Annotation>(annotation: AnnotationType): void {
    this._annotations.set((<any>annotation).prototype, annotation);
  }

  public isAnnotationPresent<AnnotationType extends Annotation>(annotationClass: ClassType<AnnotationType>): boolean {
    return null !== this.getAnnotation(annotationClass);
  }

  public getOwnAnnotation<AnnotationType extends Annotation>(annotationClass: ClassType<AnnotationType>): AnnotationType {
    return <AnnotationType>this._annotations.get(annotationClass.prototype);
  }

  public getOwnAnnotations(): Map<any, Annotation> {
    return this._annotations;
  }

  public getAnnotation<AnnotationType extends Annotation>(annotationClass: ClassType<AnnotationType>): AnnotationType {
    return this.getOwnAnnotation(annotationClass);
  }

  public getAnnotations(): Map<any, Annotation> {
    return this.getOwnAnnotations();
  }

}
