import {
  ClassType,
  PropertyKeyType,
  Metadata,
} from '@overflow/commons/core/type';

import {
  AccessibleObject,
  Field,
  Method,
} from '@overflow/commons/core/reflect';

export class Class extends AccessibleObject {
  private _clazzType: ClassType;
  private _fields: Map<PropertyKeyType, Field>;
  private _methods: Map<PropertyKeyType, Method>;

  /**
   * forClass
   */
  public static forClass(clazzType: ClassType): Class {
    return Metadata.get(LOAFER_CLASS, clazzType);
  }

  /**
   * _defineClass
   */
  public static _defineClass(clazzType: ClassType): Class {
    let clazz: Class = Metadata.get(LOAFER_CLASS, clazzType);
    if (undefined === clazz) {
      clazz = new Class(clazzType);
      Metadata.set(LOAFER_CLASS, clazz, clazzType);
    }

    return clazz;
  }

  private constructor(clazzType: ClassType) {
    super();
    this._clazzType = clazzType;
    this._fields = new Map();
    this._methods = new Map();
  }

  /**
   * _defineField
   */
  public _defineField(propertyKey: PropertyKeyType, propertyType: any): Field {
    let field: Field = this._fields.get(propertyKey);
    if (undefined === field) {
      field = new Field(this, propertyKey, propertyType);
      this._fields.set(propertyKey, field);
    }
    return field;
  }

  /**
   * _defineMethod
   */
  public _defineMethod(propertyKey: PropertyKeyType, parameterTypes: any[], returnType: any): Method {
    let method: Method = this._methods.get(propertyKey);
    if (undefined === method) {
      method = new Method(this, propertyKey, parameterTypes, returnType);
      this._methods.set(propertyKey, method);
    }
    return method;
  }


  public getOwnField(propertyKey: PropertyKeyType): Field {
    return this._fields.get(propertyKey);
  }

  public getOwnFields(): Map<PropertyKeyType, Field> {
    return this._fields;
  }

  public getField(propertyKey: PropertyKeyType): Field {
    return null;
  }

  public getFields(): Field[] {
    return null;
  }

  public getOwnMethod(propertyKey: PropertyKeyType): Method {
    return this._methods.get(propertyKey);
  }

  public getOwnMethods(): Map<PropertyKeyType, Method> {
    return this._methods;
  }

  public getMethod(propertyKey: PropertyKeyType): Method {
    return null;
  }

  public getMethods(): Method[] {
    return null;
  }

  public getName(): string {
    return null;
  }
}

/**
 * Metadata key
 * @private
 * @type {string}
 */
const LOAFER_CLASS = Symbol('loafer:class');
