import {
  PropertyKeyType,
} from '@overflow/commons/core/type';

import {
  AccessibleObject,
  Class,
  Member,
  Parameter,
 } from '@overflow/commons/core/reflect';

export abstract class Executable extends AccessibleObject implements Member {
  private _clazz: Class;
  private _parameters: Parameter[];

  protected constructor(declaringClazz: Class, parameterTypes: any[]) {
    super();

    this._clazz = declaringClazz;

    if (null === parameterTypes) {
      return;
    }
    this._parameters = [];
    for (let i = 0; i < parameterTypes.length; i++) {
      const parameterType = parameterTypes[i];
      let parameter: Parameter = new Parameter(this, parameterType, i);
      this._parameters.push(parameter);
    }
  }

  public getDeclaringClass(): Class {
    return this._clazz;
  }

  public abstract getName(): PropertyKeyType;
  /**
   * getParameterCount
   */
  public getParameterCount(): number {
    if (null === this._parameters) {
      return 0;
    }
    return this._parameters.length;
  }
  /**
   * getParameters
   */
  public getParameters(): Parameter[] {
    return this._parameters;
  }
  /**
   * getParameter
   */
  public getParameter(index: number): Parameter {
    if (null === this._parameters) {
      return null;
    }
    if (0 > index || this._parameters.length <= index) {
      return null;
    }
    return this._parameters[index];
  }
}
