import {
  Annotation,
  Class,
  Field,
  Method,
  Parameter,
} from '@overflow/commons/core/reflect';

import {
  ClassType,
  DecoratorType,
  MetadataKeyType,
  Metadata,
  TypeUtil,
} from '@overflow/commons/core/type';

import * as Constants from './constants';


export class Decorator {
  public static create = (AnnotationClass: ClassType<Annotation>) => {

    return (...handlerArgs: any[]) => {
      let annotation: Annotation = new AnnotationClass(...handlerArgs);
      let name: string = AnnotationClass.name;

      return (...decoratorArgs: any[]) => {
        let decoratorType: DecoratorType = TypeUtil.getDecoratorType(decoratorArgs);

        const [target, propertyKey, descriptorOrParameterIndex] = decoratorArgs;

        let clazz: Class = Class._defineClass(TypeUtil.getClass(target));
        let field: Field = null;
        let method: Method = null;
        let parameter: Parameter = null;

        switch(decoratorType) {
        case DecoratorType.CLASS:
          if (typeof(annotation.classDecorator) === 'undefined') {
            throw new Error(`Cannot apply @${name} decorator on Class.`);
          }

          clazz._addAnnotation(annotation);

          return annotation.classDecorator.call(annotation, target);
        case DecoratorType.PROPERTY:
          if (typeof(annotation.propertyDecorator) === 'undefined') {
            throw new Error(`Cannot apply @${name} decorator on Property.`);
          }

          field = clazz._defineField(propertyKey, Metadata.getOwnType(target, propertyKey));
          field._addAnnotation(annotation);

          return annotation.propertyDecorator.call(annotation, target, propertyKey);
        case DecoratorType.METHOD:
          if (typeof(annotation.methodDecorator) === 'undefined') {
            throw new Error(`Cannot apply @${name} decorator on Method.`);
          }

          method = clazz._defineMethod(propertyKey,
                                       Metadata.getOwnParamTypes(target, propertyKey),
                                       Metadata.getOwnReturnType(target, propertyKey));
          method._addAnnotation(annotation);

          return annotation.methodDecorator.call(annotation, target, propertyKey, descriptorOrParameterIndex);
        case DecoratorType.PARAMETER:
          if (typeof(annotation.parameterDecorator) === 'undefined') {
            throw new Error(`Cannot apply @${name} decorator on Parameter.`);
          }

          method = clazz._defineMethod(propertyKey,
                                       Metadata.getOwnParamTypes(target, propertyKey),
                                       Metadata.getOwnReturnType(target, propertyKey));
          parameter = method.getParameter(descriptorOrParameterIndex);
          parameter._addAnnotation(annotation);

          return annotation.parameterDecorator.call(annotation, target, propertyKey, descriptorOrParameterIndex);
        default:
          throw new Error(`Cannot determine decorator[@${name}] type.`);
        }
      };
    };
  }

}
